@@ -18,6 +18,7 @@ var TMP_tabDNDObserver = {
   TAB_DROP_TYPE: "application/x-moz-tabbrowser-tab",
   draggedTab: null,
   paddingLeft: 0,
+  onLastToolbar: false,
 
   init: function TMP_tabDNDObserver_init() {
     var tabBar = gBrowser.tabContainer;
@@ -76,23 +77,32 @@ var TMP_tabDNDObserver = {
       return this.orient == "horizontal" &&
         (!this.moveTabOnDragging || this.hasAttribute("multibar") ||
         aEvent.altKey);
-    }
+    };
     tabBar.useTabmixDnD = function(aEvent) {
       function checkTab(dt) {
         let tab = TMP_tabDNDObserver.getSourceNode(dt);
-        return !tab || "__tabmixDragStart" in tab;
+        return !tab || "__tabmixDragStart" in tab ||
+          TMP_tabDNDObserver.getDragType(tab) == TMP_tabDNDObserver.DRAG_TAB_TO_NEW_WINDOW;
       }
 
       return this.orient == "horizontal" &&
         (!this.moveTabOnDragging || this.hasAttribute("multibar") ||
         checkTab(aEvent.dataTransfer));
-    }
+    };
 
     this._dragOverDelay = tabBar._dragOverDelay;
     this.draglink = TabmixSvc.getString("droplink.label");
 
     // without this the Indicator is not visible on the first drag
     tabBar._tabDropIndicator.style.MozTransform = "translate(0px, 0px)";
+
+    if (Tabmix.isVersion(280)) {
+      let t = document.getElementById("TabsToolbar").parentNode.getBoundingClientRect();
+      let r = gBrowser.tabContainer.getBoundingClientRect();
+      let c = document.getElementById("content-deck").getBoundingClientRect();
+      this.onLastToolbar = Math.abs(t.bottom - r.bottom) < 2 && Math.abs(r.bottom - c.top) < 2;
+    }
+
   },
 
   get _isCustomizing() {
@@ -118,28 +128,43 @@ var TMP_tabDNDObserver = {
 
     let dt = event.dataTransfer;
     dt.mozSetDataAt(TAB_DROP_TYPE, tab, 0);
-    let uri = gBrowser.getBrowserForTab(tab).currentURI;
-    let spec = uri ? uri.spec : "about:blank";
+    let browser = tab.linkedBrowser;
 
     // We must not set text/x-moz-url or text/plain data here,
     // otherwise trying to deatch the tab by dropping it on the desktop
     // may result in an "internet shortcut"
-    dt.mozSetDataAt("text/x-moz-text-internal", spec, 0);
+    dt.mozSetDataAt("text/x-moz-text-internal", browser.currentURI.spec, 0);
 
     // Set the cursor to an arrow during tab drags.
     dt.mozCursor = "default";
 
-    let canvas = tabPreviews.capture(tab, false);
-    let offset = TabmixTabbar.position == 1 ? canvas.height + 10 : -37
-    dt.setDragImage(canvas, 0, offset);
+    // Create a canvas to which we capture the current tab.
+    // Until canvas is HiDPI-aware (bug 780362), we need to scale the desired
+    // canvas size (in CSS pixels) to the window's backing resolution in order
+    // to get a full-resolution drag image for use on HiDPI displays.
+    let windowUtils = window.getInterface(Ci.nsIDOMWindowUtils);
+    let scale = !Tabmix.isVersion(180) ? 1 :
+                windowUtils.screenPixelsPerCSSPixel / windowUtils.fullZoom;
+    let canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
+    canvas.mozOpaque = true;
+    canvas.width = 160 * scale;
+    canvas.height = 90 * scale;
+    if (!Tabmix.isVersion(340) || !gMultiProcessBrowser) {
+      // Bug 863512 - Make page thumbnails work in e10s
+      let elm = Tabmix.isVersion(360) ? browser : browser.contentWindow;
+      PageThumbs.captureToCanvas(elm, canvas);
+    }
+    let offset = (TabmixTabbar.visibleRows == 1 ? -16 : -30) * scale;
+    if (TabmixTabbar.position == 1)
+      offset = canvas.height - offset;
+    dt.setDragImage(canvas, -16 * scale, offset);
 
     // _dragData.offsetX/Y give the coordinates that the mouse should be
     // positioned relative to the corner of the new window created upon
     // dragend such that the mouse appears to have the same position
     // relative to the corner of the dragged tab.
     let clientX = function _clientX(ele) ele.getBoundingClientRect().left;
-    let tabOffsetX = clientX(tab) -
-                      clientX(gBrowser.tabs[0].pinned ? gBrowser.tabs[0] : gBrowser.tabContainer);
+    let tabOffsetX = clientX(tab) - clientX(gBrowser.tabContainer);
     tab._dragData = {
       offsetX: event.screenX - window.screenX - tabOffsetX,
       offsetY: event.screenY - window.screenY
@@ -162,8 +187,9 @@ var TMP_tabDNDObserver = {
     if (newIndex < gBrowser.tabs.length)
       left_right = this.getLeft_Right(event, newIndex, oldIndex, draggeType);
     else {
-      newIndex = draggeType != this.DRAG_TAB_IN_SAME_WINDOW && Tabmix.getOpenTabNextPref(draggeType == this.DRAG_LINK) ? tabBar.selectedIndex :
-                  gBrowser.tabs.length - 1;
+      newIndex = draggeType != this.DRAG_TAB_IN_SAME_WINDOW &&
+                 Tabmix.getOpenTabNextPref(draggeType == this.DRAG_LINK) ?
+                     tabBar.selectedIndex : gBrowser.tabs.length - 1;
       left_right = 1;
     }
 
@@ -179,7 +205,7 @@ var TMP_tabDNDObserver = {
      *     or    - we drop link that start download
      */
     if (replaceTab && !isCopy) {
-      var targetTab = gBrowser.tabs[newIndex];
+      let disAllowDrop, targetTab = gBrowser.tabs[newIndex];
       if (targetTab.getAttribute("locked") && !gBrowser.isBlankNotBusyTab(targetTab)) {
         try {
           var url = browserDragAndDrop.drop(event, { });
@@ -187,7 +213,7 @@ var TMP_tabDNDObserver = {
               /^\s*(javascript|data):/.test(url))
             url = null;
 
-          var disAllowDrop = url ? !Tabmix.ContentClick.isUrlForDownload(url) : true;
+          disAllowDrop = url ? !Tabmix.ContentClick.isUrlForDownload(url) : true;
         } catch (ex) { Tabmix.assert(ex);}
 
         if (disAllowDrop)
@@ -197,7 +223,7 @@ var TMP_tabDNDObserver = {
 
     var canDrop;
     var hideIndicator = false;
-    if (effects == "" || effects == "none" && this._isCustomizing) {
+    if (effects === "" || effects == "none" && this._isCustomizing) {
       this.clearDragmark();
       return;
     }
@@ -225,7 +251,7 @@ var TMP_tabDNDObserver = {
         this.gMsg = this.gBackupLabel;
       var statusTextFld = document.getElementById("statusbar-display");
       if (statusTextFld && statusTextFld.getAttribute("label") != this.gMsg) {
-        if (this.gBackupLabel == "")
+        if (this.gBackupLabel === "")
           this.gBackupLabel = statusTextFld.getAttribute("label");
         statusTextFld.label = this.gMsg;
         this.statusFieldChanged = true;
@@ -234,12 +260,12 @@ var TMP_tabDNDObserver = {
         let tooltip = document.getElementById("tabmix-tooltip");
         if (tooltip.state == "closed") {
           tooltip.label = this.gMsg;
-          tooltip.openPopup(document.getElementById("browser"), null, -1, -1, false, false);
+          tooltip.openPopup(document.getElementById("browser"), null, 1, 1, false, false);
         }
       }
     }
 
-    if (tabBar.overflow) {
+    if (Tabmix.tabsUtils.overflow) {
       let tabStrip = tabBar.mTabstrip;
       let ltr = Tabmix.ltr || tabStrip.orient == "vertical";
       let _scroll, targetAnonid;
@@ -253,17 +279,18 @@ var TMP_tabDNDObserver = {
       switch (targetAnonid) {
         case "scrollbutton-up":
         case "scrollbutton-up-right":
-          if (tabBar.canScrollTabsLeft)
+          if (Tabmix.tabsUtils.canScrollTabsLeft)
             _scroll = -1;
             break;
         case "scrollbutton-down":
         case "scrollbutton-down-right":
-          if (tabBar.canScrollTabsRight)
+          if (Tabmix.tabsUtils.canScrollTabsRight)
             _scroll = 1;
             break;
       }
       if (_scroll) {
-        let scrollIncrement = TabmixTabbar.isMultiRow ? Math.round(tabStrip._singleRowHeight / 6) : tabStrip.scrollIncrement;
+        let scrollIncrement = TabmixTabbar.isMultiRow ?
+            Math.round(tabStrip._singleRowHeight / 6) : tabStrip.scrollIncrement;
         tabStrip.scrollByPixels((ltr ? _scroll : -_scroll) * scrollIncrement, true);
         hideIndicator = true;
       }
@@ -305,8 +332,10 @@ var TMP_tabDNDObserver = {
     event.stopPropagation();
 
     document.getElementById("tabmix-tooltip").hidePopup();
+    /* jshint ignore:start */
     // old TreeStyleTab extension version look for isTabReorder in our code
-    var isTabReorder = draggeType == this.DRAG_TAB_IN_SAME_WINDOW
+    var isTabReorder = draggeType == this.DRAG_TAB_IN_SAME_WINDOW;
+    /* jshint ignore:end */
     var newIndex = this._getDNDIndex(event);
     var oldIndex = draggedTab ? draggedTab._tPos : -1;
     var left_right;
@@ -314,15 +343,16 @@ var TMP_tabDNDObserver = {
     if (newIndex < gBrowser.tabs.length)
       left_right = this.getLeft_Right(event, newIndex, oldIndex, draggeType);
     else {
-      newIndex = draggeType != this.DRAG_TAB_IN_SAME_WINDOW && Tabmix.getOpenTabNextPref(draggeType == this.DRAG_LINK) ? gBrowser.tabContainer.selectedIndex :
-                 gBrowser.tabs.length - 1;
+      newIndex = draggeType != this.DRAG_TAB_IN_SAME_WINDOW &&
+                 Tabmix.getOpenTabNextPref(draggeType == this.DRAG_LINK) ?
+                     gBrowser.tabContainer.selectedIndex : gBrowser.tabs.length - 1;
       left_right = 1;
     }
 
     if (draggedTab && (isCopy || draggeType == this.DRAG_TAB_IN_SAME_WINDOW)) {
       if (isCopy) {
         // copy the dropped tab (wherever it's from)
-        var newTab = gBrowser.duplicateTab(draggedTab);
+        let newTab = gBrowser.duplicateTab(draggedTab);
         gBrowser.moveTabTo(newTab, newIndex + left_right);
 
         if (draggeType == this.DRAG_TAB_TO_NEW_WINDOW || event.shiftKey)
@@ -354,12 +384,12 @@ var TMP_tabDNDObserver = {
       // swap the dropped tab with a new one we create and then close
       // it in the other window (making it seem to have moved between
       // windows)
-      newTab = gBrowser.addTab("about:blank");
+      let newTab = gBrowser.addTab("about:blank");
       var newBrowser = gBrowser.getBrowserForTab(newTab);
       // Stop the about:blank load
       newBrowser.stop();
       // make sure it has a docshell
-      newBrowser.docShell;
+      newBrowser.docShell; // jshint ignore:line
 
       gBrowser.moveTabTo(newTab, newIndex + left_right);
       gBrowser.selectedTab = newTab;
@@ -451,7 +481,7 @@ var TMP_tabDNDObserver = {
       var bo = tabBar.mTabstrip.scrollBoxObject;
       var rowHeight = TabmixTabbar.singleRowHeight;
       var endScreenY = bo.screenY + bo.height + 0.5 * rowHeight;
-      if (TabmixTabbar.position == 0) {// tabbar on the top
+      if (TabmixTabbar.position === 0) {// tabbar on the top
         if (eY < endScreenY && eY > window.screenY) {
           aEvent.stopPropagation();
           return;
@@ -544,7 +574,7 @@ var TMP_tabDNDObserver = {
   },
 
   getNewIndex: function (event) {
-    function getTabRowNumber(tab, top) tab.pinned ? 1 : gBrowser.tabContainer.getTabRowNumber(tab, top);
+    function getTabRowNumber(tab, top) tab.pinned ? 1 : Tabmix.tabsUtils.getTabRowNumber(tab, top)
     // if mX is less then the first tab return 0
     // check if mY is below the tab.... if yes go to next row
     // in the row find the closest tab by mX,
@@ -554,24 +584,26 @@ var TMP_tabDNDObserver = {
     var tabs = gBrowser.visibleTabs;
     var numTabs = tabs.length;
     if (!tabBar.hasAttribute("multibar")) {
-      for (let i = event.target.localName == "tab" ? TMP_TabView.getIndexInVisibleTabsFromTab(event.target) : 0; i < numTabs; i++) {
+      let i = event.target.localName == "tab" ?
+          Tabmix.visibleTabs.indexOf(event.target) : 0;
+      for (; i < numTabs; i++) {
         let tab = tabs[i];
         if (Tabmix.compare(mX, Tabmix.itemEnd(tab, Tabmix.ltr), Tabmix.ltr))
           return i;
       }
     }
     else {
-      let top = tabBar.topTabY;
+      let topY = Tabmix.tabsUtils.topTabY;
       for (let i = 0; i < numTabs; i++) {
         let tab = tabs[i];
-        let thisRow = getTabRowNumber(tab, top);
+        let thisRow = getTabRowNumber(tab, topY);
         if (mY >= tab.boxObject.screenY + tab.boxObject.height) {
-          while (i < numTabs - 1 && getTabRowNumber(tabs[i+1], top) == thisRow)
+          while (i < numTabs - 1 && getTabRowNumber(tabs[i+1], topY) == thisRow)
             i++;
         }
         else if (Tabmix.compare(mX, Tabmix.itemEnd(tab, Tabmix.ltr), Tabmix.ltr))
           return i;
-        else if (i == numTabs - 1 || getTabRowNumber(tabs[i+1], top) != thisRow)
+        else if (i == numTabs - 1 || getTabRowNumber(tabs[i+1], topY) != thisRow)
           return i;
       }
     }
@@ -627,22 +659,21 @@ var TMP_tabDNDObserver = {
    this.clearDragmark();// clear old dragmark if one exist
 
    if (!Tabmix.prefs.getBoolPref("useFirefoxDragmark")) {
-      var sameRow = newIndex != 0 && newIndex != gBrowser.tabs.length &&
+      var sameRow = newIndex !== 0 && newIndex != gBrowser.tabs.length &&
             TabmixTabbar.inSameRow(gBrowser.tabs[newIndex-1], gBrowser.tabs[newIndex]);
-      if (sameRow || left_right==0)
+      if (sameRow || left_right === 0)
          this.setDragmarkAttribute(gBrowser.tabs[newIndex], "atLeft");
-      if (sameRow || left_right==1)
+      if (sameRow || left_right == 1)
          this.setDragmarkAttribute(gBrowser.tabs[newIndex-1], "atRight");
    }
    else {
       // code for firefox indicator
       var ind = gBrowser.tabContainer._tabDropIndicator;
       var minMargin, maxMargin, newMargin;
-      var tabBoxObject, tabRect;
+      var tabRect;
       var ltr = Tabmix.ltr;
       let scrollRect = gBrowser.tabContainer.mTabstrip.scrollClientRect;
       let rect = gBrowser.tabContainer.getBoundingClientRect();
-      let scrollMode = TabmixTabbar.scrollButtonsMode;
       minMargin = scrollRect.left - rect.left - this.paddingLeft;
       maxMargin = Math.min(minMargin + scrollRect.width, scrollRect.right);
       if (!ltr)
@@ -650,22 +681,34 @@ var TMP_tabDNDObserver = {
 
       tabRect = gBrowser.tabs[index].getBoundingClientRect();
       if (ltr)
-         newMargin = tabRect.left - rect.left  + (left_right == 1 ? tabRect.width + this.LinuxMarginEnd: 0) - this.paddingLeft;
+         newMargin = tabRect.left - rect.left  +
+                     (left_right == 1 ? tabRect.width + this.LinuxMarginEnd: 0) -
+                     this.paddingLeft;
       else
-         newMargin = rect.right - tabRect.left - (left_right == 0 ? tabRect.width + this.LinuxMarginEnd : 0) - this.paddingLeft;
+         newMargin = rect.right - tabRect.left -
+                     (left_right === 0 ? tabRect.width + this.LinuxMarginEnd : 0) -
+                     this.paddingLeft;
 
-      ///XXX fix min/max x margin when in one row the drag mark is visible after the arrow when the last tab is partly visible
+      ///XXX fix min/max x margin when in one row the drag mark is visible after
+      ///XXX the arrow when the last tab is partly visible
       ///XXX look like the same is happen with Firefox
-      var newMarginY;
+      var newMarginY, fixMargin;
       if (TabmixTabbar.position == 1) {
         newMarginY = tabRect.bottom - ind.parentNode.getBoundingClientRect().bottom;
-        if (document.getElementById("addon-bar").collapsed)
-          ind.style.marginBottom = "0px";
-        else
-          ind.style.removeProperty("margin-bottom");
+        let addOnBar = document.getElementById("addon-bar");
+        fixMargin = (Tabmix.isVersion(280) || addOnBar && addOnBar.collapsed) &&
+          (Math.abs(newMarginY) < 0.5);
       }
-      else
+      else {
         newMarginY = tabRect.bottom - rect.bottom;
+        fixMargin = this.onLastToolbar && (Math.abs(newMarginY) < 0.5);
+      }
+      // make indicator visible
+      if (fixMargin)
+        ind.style.marginBottom = "1px";
+      else
+        ind.style.removeProperty("margin-bottom");
+
       this.setFirefoxDropIndicator(true);
       newMargin += ind.clientWidth / 2;
       if (!ltr)
@@ -679,14 +722,14 @@ var TMP_tabDNDObserver = {
   },
 
   clearDragmark: function minit_clearDragmark() {
-    if (this.dragmarkindex == null)
+    if (this.dragmarkindex === null)
       return;
 
     if (!Tabmix.prefs.getBoolPref("useFirefoxDragmark")) {
       var index = this.dragmarkindex.newIndex;
       if (index != gBrowser.tabs.length && gBrowser.tabs[index].hasAttribute("dragmark"))
          this.removetDragmarkAttribute(gBrowser.tabs[index]);
-      if (index != 0 && gBrowser.tabs[index-1].hasAttribute("dragmark"))
+      if (index !== 0 && gBrowser.tabs[index-1].hasAttribute("dragmark"))
          this.removetDragmarkAttribute(gBrowser.tabs[index-1]);
     }
     else
@@ -714,7 +757,7 @@ var TMP_tabDNDObserver = {
     var dt = aEvent.dataTransfer;
     // Disallow dropping multiple items
     if (dt.mozItemCount > 1)
-      return dt.effectAllowed = "none";
+      return (dt.effectAllowed = "none");
 
     var types = dt.mozTypesAt(0);
     // move or copy tab
@@ -725,20 +768,22 @@ var TMP_tabDNDObserver = {
         // and vice versa.
         (Tabmix.isVersion(200) && PrivateBrowsingUtils.isWindowPrivate(window) !=
             PrivateBrowsingUtils.isWindowPrivate(sourceNode.ownerDocument.defaultView))){
-        return dt.effectAllowed = "none";
+        return (dt.effectAllowed = "none");
       }
 
       if (Tabmix.isVersion(310) && window.gMultiProcessBrowser !=
           sourceNode.ownerDocument.defaultView.gMultiProcessBrowser)
-        return dt.effectAllowed = "none";
+        return (dt.effectAllowed = "none");
 
-      return dt.effectAllowed = "copyMove";
+      let copyModifier = Tabmix.isVersion(390) &&
+          gBrowser.AppConstants.platform == "macosx" ? aEvent.altKey : aEvent.ctrlKey;
+      return (dt.effectAllowed = copyModifier ? "copy" : "move");
     }
 
     if (browserDragAndDrop.canDropLink(aEvent)) {
-      return dt.effectAllowed = dt.dropEffect = "link";
+      return (dt.effectAllowed = dt.dropEffect = "link");
     }
-    return dt.effectAllowed = "none";
+    return (dt.effectAllowed = "none");
   },
 
   getSourceNode: function TMP_getSourceNode(aDataTransfer) {
@@ -748,10 +793,10 @@ var TMP_tabDNDObserver = {
     return null;
   }
 
-} // TMP_tabDNDObserver end
+}; // TMP_tabDNDObserver end
 
 var TMP_undocloseTabButtonObserver = {
-  onDragOver: function (aEvent, aFlavour, aDragSession) {
+  onDragOver: function (aEvent) {
     var dt = aEvent.dataTransfer;
     var sourceNode = TMP_tabDNDObserver.getSourceNode(dt) || this.NEW_getSourceNode(dt);
     if (!sourceNode || sourceNode.localName != "tab") {
@@ -776,7 +821,7 @@ var TMP_undocloseTabButtonObserver = {
     return true;
   },
 
-  onDragExit: function (aEvent, aDragSession) {
+  onDragExit: function (aEvent) {
     if (aEvent.target.hasAttribute("dragover")) {
       var statusTextFld = document.getElementById("statusbar-display");
       if (statusTextFld)
@@ -788,7 +833,7 @@ var TMP_undocloseTabButtonObserver = {
     }
   },
 
-  onDrop: function (aEvent, aXferData, aDragSession) {
+  onDrop: function (aEvent) {
     var dt = aEvent.dataTransfer;
     var sourceNode = TMP_tabDNDObserver.getSourceNode(dt) || this.NEW_getSourceNode(dt);
     if (sourceNode && sourceNode.localName == "tab")
@@ -806,7 +851,7 @@ var TMP_undocloseTabButtonObserver = {
       node = node.parentNode;
     return node && node.localName == "tab" ? node : null;
   }
-}
+};
 
 /* ::::::::::     miscellaneous     :::::::::: */
 
@@ -815,14 +860,14 @@ Tabmix.goButtonClick = function TMP_goButtonClick(aEvent) {
     gBrowser.duplicateTab(gBrowser.mCurrentTab);
   else if (aEvent.button != 2)
     gURLBar.handleCommand(aEvent);
-}
+};
 
 Tabmix.loadTabs = function TMP_loadTabs(aURIs, aReplace) {
   let bgLoad = Services.prefs.getBoolPref("browser.tabs.loadInBackground");
   try {
     gBrowser.loadTabs(aURIs, bgLoad, aReplace);
   } catch (ex) { }
-}
+};
 
 Tabmix.whereToOpen = function TMP_whereToOpen(pref, altKey) {
    var aTab = gBrowser.mCurrentTab;
@@ -841,14 +886,14 @@ Tabmix.whereToOpen = function TMP_whereToOpen(pref, altKey) {
       openTabPref = (altKey ^ openTabPref) == 1;
    }
    return { inNew: !isBlankTab && (isLockTab || openTabPref), lock: isLockTab };
-}
+};
 
 Tabmix.getStyle = function TMP_getStyle(aObj, aStyle) {
   try {
     return parseInt(window.getComputedStyle(aObj, null)[aStyle]) || 0;
   } catch (ex) {this.assert(ex);}
   return 0;
-}
+};
 
 // sometimes context popup stay "open", we hide it manually.
 Tabmix.hidePopup = function TMP_hidePopup(aPopupMenu) {
@@ -860,24 +905,15 @@ Tabmix.hidePopup = function TMP_hidePopup(aPopupMenu) {
       }
       node = node.parentNode;
    }
-}
-
-var TMP_TabView = {
-  __noSuchMethod__: function(id, args) {
-    if (!this.installed)
-      return;
-    if (typeof TabView[id] == "function")
-      TabView[id].apply(TabView, args);
-    else
-      Tabmix.log("Error " + id + " is not exist in TabView", true);
-  },
+};
 
+var TMP_TabView = { /* jshint ignore: line */
   get installed() {
     delete this.installed;
     let installed = typeof TabView == "object";
     if (installed)
       Services.scriptloader.loadSubScript("chrome://tabmixplus/content/minit/tabView.js", window);
-    return this.installed = installed;
+    return (this.installed = installed);
   },
 
   checkTabs: function (tabs) {
@@ -892,22 +928,6 @@ var TMP_TabView = {
     return firstTab;
   },
 
-  previousVisibleSibling: function (aTab) {
-    var tabs = gBrowser.visibleTabs;
-    var index = tabs.indexOf(aTab);
-    if (--index > -1)
-      return tabs[index];
-    return null;
-  },
-
-  nextVisibleSibling: function (aTab) {
-    var tabs = gBrowser.visibleTabs;
-    var index = tabs.indexOf(aTab);
-    if (index > -1 && ++index < tabs.length)
-      return tabs[index];
-    return null;
-  },
-
   // includung _removingTabs
   currentGroup: function () {
     return Array.filter(gBrowser.tabs, function(tab) !tab.hidden);
@@ -926,12 +946,8 @@ var TMP_TabView = {
     if (aTab)
       return gBrowser.visibleTabs.indexOf(aTab);
     return -1;
-  },
-
-  getIndexInVisibleTabsFrom_tPos: function (aIndex) {
-    return this.getIndexInVisibleTabsFromTab(gBrowser.tabs.item(aIndex));
   }
-}
+};
 
 Tabmix.navToolbox = {
   customizeStarted: false,
@@ -962,7 +978,7 @@ Tabmix.navToolbox = {
             gTMPprefObserver.showReloadEveryOnReloadButton();
         }
       }.bind(this)
-    }
+    };
     CustomizableUI.addListener(this.listener);
   },
 
@@ -1066,7 +1082,7 @@ Tabmix.navToolbox = {
     // onblur attribut reset each time we exit ToolboxCustomize
     var blur = gURLBar.getAttribute("onblur") || "";
     if (blur.indexOf("Tabmix.urlBarOnBlur") == -1)
-      gURLBar.setAttribute("onblur", blur + "Tabmix.urlBarOnBlur();")
+      gURLBar.setAttribute("onblur", blur + "Tabmix.urlBarOnBlur();");
 
     let obj = gURLBar, fn;
     // Fix incompatibility with Omnibar (O is not defined)
@@ -1082,11 +1098,10 @@ Tabmix.navToolbox = {
     else if ("urlDot" in window && "handleCommand2" in gURLBar)
       fn = "handleCommand2";
     else
-      fn = "handleCommand"
+      fn = "handleCommand";
 
     // Fix incompatibility with https://addons.mozilla.org/en-US/firefox/addon/url-fixer/
     if ("urlfixerOldHandler" in gURLBar.handleCommand) {
-      _handleCommand = gURLBar.handleCommand.urlfixerOldHandler.toString();
       obj = gURLBar.handleCommand;
       fn = "urlfixerOldHandler";
     }
@@ -1101,19 +1116,20 @@ Tabmix.navToolbox = {
       return;
 
     // InstantFox extension uses old version of gURLBar.handleCommand until Firefox 25
-    let instantFox = !Tabmix.isVersion(250) && typeof InstantFox == "object";
+    let instantFox = !Tabmix.isVersion(250, 250) && typeof InstantFox == "object";
 
     // we don't do anything regarding IeTab and URL Suffix extensions
     _handleCommand = Tabmix.changeCode(obj, "gURLBar." + fn, {silent: this.urlBarInitialized})._replace(
       '{',
       '{\
-       if (gBrowser.tabmix_tab) {\
+       if (Tabmix.selectedTab) {\
          Tabmix.selectedTab = null;\
          Tabmix.userTypedValue = "";\
        }'
     )._replace(
       'if (isMouseEvent || altEnter) {',
-      'let loadNewTab = Tabmix.whereToOpen("extensions.tabmix.opentabfor.urlbar", altEnter).inNew && !(/^ *javascript:/.test(url));\
+      'let loadNewTab = Tabmix.whereToOpen("extensions.tabmix.opentabfor.urlbar", altEnter).inNew &&\
+           !(/^ *javascript:/.test(url));\
        if (isMouseEvent || altEnter || loadNewTab) {', {check: !instantFox}
     )._replace(
       // always check whereToOpenLink except for alt to catch also ctrl/meta
@@ -1140,7 +1156,8 @@ Tabmix.navToolbox = {
          let tabEmpty = !isTabEmpty(gBrowser.selectedTab);\
          let altEnter = !isMouseEvent && aTriggeringEvent && aTriggeringEvent.altKey && !tabEmpty;\
          let loadNewTab = InstantFoxModule.currentQuery && InstantFoxModule.openSearchInNewTab && !tabEmpty ||\
-                          Tabmix.whereToOpen("extensions.tabmix.opentabfor.urlbar", altEnter).inNew && !(/^ *javascript:/.test(url));\
+                          Tabmix.whereToOpen("extensions.tabmix.opentabfor.urlbar", altEnter).inNew &&\
+                          !(/^ *javascript:/.test(url));\
          let inBackground = Tabmix.prefs.getBoolPref("loadUrlInBackground");\
          $&'
       )._replace(
@@ -1152,7 +1169,7 @@ Tabmix.navToolbox = {
       )._replace(
         'inBackground: false',
         'inBackground: inBackground'
-      )
+      );
     }
     _handleCommand.toCode();
 
@@ -1171,6 +1188,17 @@ Tabmix.navToolbox = {
     }
   },
 
+  whereToOpenSearch: function(aWhere) {
+    var tab = gBrowser.selectedTab;
+    var isBlankTab = gBrowser.isBlankNotBusyTab(tab);
+    var isLockTab = !isBlankTab && tab.hasAttribute("locked");
+    if (aWhere == "current" && isLockTab)
+      aWhere = "tab";
+    else if (/^tab/.test(aWhere) && isBlankTab)
+      aWhere = "current";
+    return aWhere;
+  },
+
   initializeSearchbar: function TMP_navToolbox_initializeSearchbar() {
     var searchbar = document.getElementById("searchbar");
     if (!searchbar)
@@ -1195,24 +1223,41 @@ Tabmix.navToolbox = {
     let organizeSE = "organizeSE" in window && "doSearch" in window.organizeSE;
     let [obj, fn] = searchLoadExt ? [esteban_torres.searchLoad_Options, "MOZdoSearch"] :
                                     [organizeSE ? window.organizeSE : searchbar, "doSearch"];
-    if (obj[fn].toString().indexOf("tabmixArg") > -1)
+    let fnString = obj[fn].toString();
+    if (/Tabmix/.test(fnString))
       return;
+
+    // Personas Interactive Theme Engine 1.6.5
+    let pIte = fnString.indexOf("BTPIServices") > -1;
+
+    let $LF = '\n          ';
     Tabmix.changeCode(obj, "searchbar." + fn)._replace(
-      /(openUILinkIn[^\(]*\([^\)]+)(\))/,
-      '$1, null, tabmixArg$2'
-    )._replace(
-      'openUILinkIn',
-      '  var tabmixArg = {backgroundPref: "extensions.tabmix.loadSearchInBackground"};' +
-      '  var isBlankTab = gBrowser.isBlankNotBusyTab(gBrowser.mCurrentTab);' +
-      '  var isLockTab = !isBlankTab && gBrowser.mCurrentTab.hasAttribute("locked");' +
-      '  if (aWhere == "current" && isLockTab)' +
-      '    aWhere = "tab";' +
-      '  else if ((/^tab/).test(aWhere) && isBlankTab)' +
-      '    aWhere = "current";' +
+      /let params|openUILinkIn/,
+      'aWhere = Tabmix.navToolbox.whereToOpenSearch(aWhere);' + $LF +
       '$&'
+    )._replace(
+      /openUILinkIn\(.*\);/,
+      'let params = {' + $LF +
+      '  postData: submission.postData,' + $LF +
+      '  inBackground: Tabmix.prefs.getBoolPref("loadSearchInBackground")' + $LF +
+      '};' + $LF +
+      'openUILinkIn(submission.uri.spec, aWhere, params);',
+      {check: !Tabmix.isVersion(350)}
+    )._replace(
+      'inBackground: aWhere == "tab-background"',
+      '$& ||' + $LF +
+      '                Tabmix.prefs.getBoolPref("loadSearchInBackground")',
+      {check: Tabmix.isVersion(350)}
     )._replace(
       'var loadInBackground = prefs.getBoolPref("loadBookmarksInBackground");',
-      'var loadInBackground = Tabmix.prefs.getBoolPref("loadSearchInBackground");', {check: !searchLoadExt && organizeSE}
+      'var loadInBackground = aWhere == "tab-background" || Tabmix.prefs.getBoolPref("loadSearchInBackground");',
+      {check: !searchLoadExt && organizeSE}
+    )._replace(
+      /searchbar\.currentEngine/g,
+      'this.currentEngine', {check: pIte}
+    )._replace(
+      /BTPIServices/g,
+      'Services', {check: pIte}
     ).toCode();
   },
 
@@ -1228,7 +1273,7 @@ Tabmix.navToolbox = {
       alltabsPopup.__ensureElementIsVisible = function () {
         let scrollBox = document.getAnonymousElementByAttribute(this, "class", "popup-internal-box");
         scrollBox.ensureElementIsVisible(gBrowser.mCurrentTab.mCorrespondingMenuitem);
-      }
+      };
       alltabsPopup.addEventListener("popupshown", alltabsPopup.__ensureElementIsVisible, false);
 
       // alltabs-popup fix visibility for multi-row
@@ -1246,7 +1291,9 @@ Tabmix.navToolbox = {
      * restore tabmix-tabs-closebutton and new-tab-button position.
      */
     this.setScrollButtons();
-    this.setCloseButtonPosition();
+    try {
+      this.setCloseButtonPosition();
+    } catch(ex) { }
     gTMPprefObserver.changeNewTabButtonSide(Tabmix.prefs.getIntPref("newTabButton.position"));
     this.setScrollButtons(false, true);
 
@@ -1269,7 +1316,7 @@ Tabmix.navToolbox = {
       next.parentNode.insertBefore(box, next);
       if (!onlyPosition) {
         let useTabmixButtons = TabmixTabbar.scrollButtonsMode > TabmixTabbar.SCROLL_BUTTONS_LEFT_RIGHT;
-        gBrowser.tabContainer.mTabstrip.updateScrollButtons(useTabmixButtons);
+        Tabmix.tabsUtils.updateScrollButtons(useTabmixButtons);
       }
       return;
     }
@@ -1278,7 +1325,7 @@ Tabmix.navToolbox = {
 
     if (!onlyPosition) {
       let useTabmixButtons = TabmixTabbar.scrollButtonsMode > TabmixTabbar.SCROLL_BUTTONS_LEFT_RIGHT;
-      gBrowser.tabContainer.mTabstrip.updateScrollButtons(useTabmixButtons);
+      Tabmix.tabsUtils.updateScrollButtons(useTabmixButtons);
     }
   },
 
@@ -1286,7 +1333,6 @@ Tabmix.navToolbox = {
   setCloseButtonPosition: function() {
    if (this._closeButtonInitialized)
       return;
-    this._closeButtonInitialized = true;
 
     if (!Tabmix.isVersion(310))
       return;
@@ -1301,6 +1347,7 @@ Tabmix.navToolbox = {
     // try to restore button position from tabs-closebutton position
     // if item with tabs-closebutton id exist, some other extension add it
     else if (!document.getElementById("tabs-closebutton")) {
+      // will throw if called too early (before placements have been fetched)
       let currentset = CustomizableUI.getWidgetIdsInArea("TabsToolbar");
       let position = currentset.indexOf("tabs-closebutton");
       if (position > -1) {
@@ -1308,11 +1355,12 @@ Tabmix.navToolbox = {
         CustomizableUI.moveWidgetWithinArea("tabmix-tabs-closebutton", position);
       }
     }
+    this._closeButtonInitialized = true;
   }
 
-}
+};
 
 Tabmix.getPlacement = function(id) {
   let placement = CustomizableUI.getPlacementOfWidget(id);
   return placement ? placement.position : null;
-}
+};
